
# q-expansion of newform 156.2.k.e, downloaded from the LMFDB on 28 April 2024.

# We generate the q-expansion using the Hecke eigenvalues a_p at the primes.
# Each a_p is given as a linear combination
# of the following basis for the coefficient ring.

def make_data():

    from sage.all import prod, floor, prime_powers, gcd, QQ, primes_first_n, next_prime, RR

    def discrete_log(elts, gens, mod):
        # algorithm 2.2, page 16 of https://arxiv.org/abs/0903.2785
        def table_gens(gens, mod):
            T = [1]
            n = len(gens)
            r = [None]*n
            s = [None]*n
            for i in range(n):
                beta = gens[i]
                r[i] = 1
                N = len(T)
                while beta not in T:
                    for Tj in T[:N]:
                        T.append((beta*Tj) % mod)
                    beta = (beta*gens[i]) % mod
                    r[i] += 1
                s[i] = T.index(beta)
            return T, r, s
        T, r, s = table_gens(gens, mod)
        n = len(gens)
        N = [ prod(r[:j]) for j in range(n) ]
        Z = lambda s: [ (floor(s/N[j]) % r[j]) for j in range(n)]
        return [Z(T.index(elt % mod)) for elt in elts]
    def extend_multiplicatively(an):
        for pp in prime_powers(len(an)-1):
            for k in range(1, (len(an) - 1)//pp + 1):
                if gcd(k, pp) == 1:
                    an[pp*k] = an[pp]*an[k]
    from sage.all import PolynomialRing, NumberField, ZZ
    R = PolynomialRing(QQ, "x")
    f = R(poly_data)
    K = NumberField(f, "a")
    betas = [K([c/ZZ(den) for c in num]) for num, den in basis_data]
    convert_elt_to_field = lambda elt: sum(c*beta for c, beta in zip(elt, betas))
    # convert aps to K elements
    primes = primes_first_n(len(aps_data))
    good_primes = [p for p in primes if not p.divides(level)]
    aps = map(convert_elt_to_field, aps_data)
    if not hecke_ring_character_values:
        # trivial character
        char_values = dict(zip(good_primes, [1]*len(good_primes)))
    else:
        gens = [elt[0] for elt in hecke_ring_character_values]
        gens_values = [convert_elt_to_field(elt[1]) for elt in hecke_ring_character_values]
        char_values = dict([(
            p,prod(g**k for g, k in zip(gens_values, elt)))
            for p, elt in zip(good_primes, discrete_log(good_primes, gens, level))
            ])
    an_list_bound = next_prime(primes[-1])
    an = [0]*an_list_bound
    an[1] = 1
    
    from sage.all import PowerSeriesRing
    PS = PowerSeriesRing(K, "q")
    for p, ap in zip(primes, aps):
        if p.divides(level):
            euler_factor = [1, -ap]
        else:
            euler_factor = [1, -ap, p**(weight - 1) * char_values[p]]
        k = RR(an_list_bound).log(p).floor() + 1
        foo = (1/PS(euler_factor)).padded_list(k)
        for i in range(1, k):
            an[p**i] = foo[i]
    extend_multiplicatively(an)
    return PS(an)
level = 156
weight = 2
poly_data = [32, 0, 16, -8, -6, -6, -3, -2, 2, 0, 1]

# The entries in the following list give a basis for the
# coefficient ring in terms of a root of the defining polynomial above.
# Each line consists of the coefficients of the numerator, and a denominator.
basis_data  = [[[1, 0, 0, 0, 0, 0, 0, 0, 0, 0], 1], [[0, 1, 0, 0, 0, 0, 0, 0, 0, 0], 1], [[80, 8, 28, 6, -8, -9, -2, -6, 2, -1], 16], [[24, -12, 10, 0, -1, 1, 0, -2, 1, -1], 4], [[-48, -88, 12, -2, 16, 19, 6, 2, 2, -5], 16], [[80, -24, 4, -14, -16, -7, 2, -2, 6, 1], 16], [[96, -72, 16, -26, -14, 1, 6, -2, 8, -3], 16], [[-80, 24, 12, 14, 16, 7, -2, 2, -6, -1], 16], [[48, 8, -24, -14, -22, -9, 2, 2, 4, 3], 16], [[-64, -88, 0, -6, 22, 23, 10, 2, 0, -5], 16]]

hecke_ring_character_values = [[79, [-1, 0, 0, 0, 0, 0, 0, 0, 0, 0]], [53, [1, 0, 0, 0, 0, 0, 0, 0, 0, 0]], [145, [0, 0, 0, 0, 0, 0, 0, 0, -1, 0]]]
aps_data = [[0, -1, 0, 0, 0, 0, 0, 0, 0, 0], [0, 0, 0, 0, 0, 0, 0, 0, 1, 0], [0, 0, 1, 1, 0, 1, 0, 1, 0, 0], [0, 0, 0, -1, 1, -1, 1, 0, 0, 0], [1, 1, 1, 1, 0, 0, -1, 0, -1, 0], [1, 0, 1, 0, 1, 0, -1, 1, 2, 0], [0, -1, 0, -1, -1, 2, 0, -1, 2, 0], [2, 0, 1, -1, -1, 0, 0, 0, 2, 1], [-2, 1, 1, -1, -1, -1, -1, -1, 0, 1], [-2, -1, 0, -1, -1, -2, 0, -1, 0, 0], [-2, 1, -1, 2, 0, 0, 2, -1, -2, -1], [-1, 1, -2, 1, 1, 0, 0, 1, -1, 0], [2, 0, -1, -3, 0, -1, 2, 1, -2, -2], [4, -2, 0, -2, 0, -2, 0, 0, 0, -2], [1, 1, 1, -1, 0, 0, 1, 2, -1, -2], [-2, -1, 0, 1, -1, -2, 2, -3, 0, 2], [1, -3, -3, -1, -2, 2, 1, 0, -1, 0], [0, 0, -2, 2, -2, 0, 0, 0, 0, 2], [0, 0, -3, 3, 1, -2, 0, 2, 0, -1], [-5, 1, -2, 3, 0, -1, 2, 0, -5, -1], [1, -2, 4, -2, -4, -2, 0, 0, 1, 2], [0, -2, 0, -4, -2, 4, 2, 0, 0, 2], [1, 1, -4, 3, 2, -1, 0, 2, 1, -1], [0, 1, 1, -4, 1, 3, -2, 0, 0, 2], [1, -2, -2, -4, 2, -2, 4, 0, -1, 0], [0, -2, 2, 4, 2, -2, -4, 0, 2, 0], [0, 0, 0, 2, 2, 2, 2, 0, 0, 0], [0, 3, -5, -1, 3, -1, -1, -3, 2, -1], [3, -1, 0, 1, -3, 4, 0, 3, -3, -2], [2, 0, 2, -2, 0, -2, 0, -2, 0, 0], [0, 5, 5, -5, -1, -1, -5, -1, 0, 1], [0, -3, 1, -3, 1, 1, 1, 3, 6, 5], [-2, 3, -5, 4, 3, 1, 2, 0, -2, -2], [0, 1, -1, -3, -1, 1, 3, 1, -4, 1], [-2, 2, 1, 3, -2, 1, -4, -1, 2, 0], [-2, 4, 4, 5, 1, -1, -5, -2, 2, 2], [4, 3, 5, -5, 1, -1, -3, -1, 0, -1], [2, -3, 0, 4, 0, 3, -1, -1, -2, 4], [1, 1, -3, -5, 4, -8, 1, -6, -1, 2], [0, 2, 2, 4, 2, -6, 0, 4, -6, 0], [0, 5, 4, -3, 1, 2, -4, 1, 0, 0], [0, 1, -1, -3, -3, 3, 3, -1, -2, -1], [0, -1, 0, -3, -1, 2, 2, 1, -4, 2], [1, -2, 0, 4, 0, -2, 2, -2, 1, -4], [-4, -4, -1, -1, 0, 3, 4, 3, 4, 0], [-12, -3, -1, -1, -1, -3, 1, -1, 0, -1], [0, 2, -4, -2, 2, 0, 0, -2, 12, 0], [-2, 1, -7, 8, 2, -4, 2, 3, -2, -3], [3, -7, 4, 3, -6, -9, 0, 2, 3, -1], [-9, -3, 4, -3, -1, 0, -2, -1, -9, 0], [0, -3, 0, -5, -3, 6, 2, -1, -6, 2], [-3, -5, 4, -1, 0, -1, -2, -2, -3, -3], [-5, 0, 2, 0, -6, -4, 2, 2, -5, 4], [10, 4, -3, 2, -2, 5, -5, 6, 0, 1], [0, -3, 4, 3, -3, 2, -2, -1, -10, -2], [0, 4, -1, 2, 4, -7, 1, 4, -6, 1], [-10, 7, 2, 3, -1, 4, -2, -1, 0, 6], [12, 2, -2, -5, -3, -1, 1, 2, -12, -6], [0, 8, -8, -2, 4, -4, 2, -2, 0, -2], [2, 1, -7, 4, 5, -1, -2, 4, 2, -2], [4, 3, -1, 1, 1, 5, -3, 5, 0, -1], [6, 1, 1, -2, -3, -1, 0, 2, 6, 4], [0, 1, 6, 8, 2, 3, -3, 1, 0, 4], [10, 5, -3, 7, -1, 7, -1, 3, 0, 5], [2, 5, 1, 1, 1, 5, -3, 3, 0, 1], [0, 2, -3, -1, 2, -7, -4, -9, 0, 4], [-8, 4, -1, 3, -3, 0, 6, -2, -8, 1], [0, 3, -5, -3, 5, -3, 1, 1, 8, 3], [0, 2, -2, 2, 2, -2, -2, -2, -4, -2], [1, -1, -2, 7, -1, -4, 4, -1, 1, -4], [10, 4, 1, -5, 0, -3, 2, 3, -10, -6], [-13, -3, -5, -5, -2, 0, 3, 0, 13, -2], [0, -9, 11, 7, -7, 5, -5, -1, -4, -3], [-6, -11, -9, 3, -3, -5, 5, 1, 0, -3], [-2, 8, -1, -7, 5, 12, 0, -4, -2, 3], [11, -3, 2, -5, 6, 5, -6, -2, 11, -3], [0, -2, 4, 6, 2, -4, -4, 2, 6, 0], [9, -3, 4, -3, 1, 2, -2, -3, 9, -2], [-8, -3, 7, -2, -3, 1, 2, -6, -8, -2], [1, -6, -2, 6, -4, 8, -2, 0, -1, 4], [0, 5, -7, 1, 5, -3, -3, -5, 10, -3], [5, -1, 2, 3, 3, 0, 0, -1, -5, 4], [15, 1, 2, -5, -2, 1, -2, 2, 15, 5], [0, 2, 6, 10, 0, -8, -2, 4, 10, -4], [4, 0, -2, 6, -2, 0, 4, -4, 0, 6], [0, 9, -9, -7, 1, -1, 7, -1, 2, -1], [4, 5, 3, -10, 5, 13, -2, -6, 4, 2], [7, 6, 8, 6, 0, 2, -4, 4, -7, -2], [4, 2, 5, -1, 6, -3, 4, 3, -4, 0], [8, 2, 2, 3, 1, -1, -3, -2, -8, 2], [-30, 0, 1, 6, 2, 1, 7, -6, 0, 5], [9, -3, -1, -1, 2, 0, 3, 0, -9, 2], [2, 5, -7, 6, 0, -2, 4, 3, 2, 1], [-2, 0, -1, 6, 6, 7, 5, 2, 0, -1], [-8, 0, 9, 1, -13, -4, 10, -6, -8, 3], [0, -4, 3, 10, 8, -7, -11, 0, 2, 1], [-12, -2, 3, 5, 2, 3, 0, 1, 12, 4], [14, -2, -2, -4, -2, -2, -4, 4, 0, -4], [0, 5, -1, 1, -1, -3, 3, 1, -8, -3], [-1, 5, -6, -7, 11, 10, -8, 3, -1, 2], [0, -5, 11, 11, -3, -3, -5, 3, 12, -3], [2, -7, 13, -6, -7, -1, 0, -6, 2, 0], [0, 3, 0, 7, 3, 6, 4, -1, 0, 4], [0, 3, -6, -7, 3, 0, 4, 1, 10, 4], [0, -3, -1, -5, -5, -7, -3, -1, 0, -1], [1, 2, -2, 2, 2, -6, -6, -10, -1, 6], [-17, 1, 0, -1, 0, 1, 0, 0, -17, 1], [-8, 5, -3, 4, 1, 3, 6, -4, -8, -2], [0, -4, 7, 2, -4, 1, 1, 4, 6, 1], [2, -7, 5, -7, 1, -11, 5, -9, 0, -3], [0, 1, -1, -3, -5, 5, 3, -3, -12, -3], [1, -7, -12, -11, -1, -4, 6, -5, -1, 0], [6, 6, -1, -1, -6, -1, -6, -1, -6, -6], [4, -5, -2, 0, -2, 5, 3, 3, -4, 0], [-14, -4, -2, -1, -1, 3, 3, 2, 14, 0], [0, 5, -6, -11, 1, 0, 10, 5, 2, 6], [4, 6, -5, -3, 3, 4, -2, 4, 4, 5], [8, -6, -4, -2, -2, -4, 0, 2, 0, -4], [-2, 9, 2, 1, -3, 4, -6, 1, 0, 6], [0, 2, 2, 2, -6, 2, 2, -2, 4, -6], [11, 7, -4, -5, 7, 10, -2, -1, 11, 2], [0, 7, -9, -7, -1, 3, 5, -5, -18, -3], [18, -3, -2, -5, -3, -4, -4, 3, 0, -4], [11, 5, -1, -3, -2, -4, -3, -2, -11, -4], [-24, 2, 3, -1, -7, -2, 4, 0, -24, 5], [0, 0, 2, 0, -4, 2, 2, 0, 6, -2], [-3, 5, -8, 1, 7, 4, -2, 3, -3, 0], [-22, 1, -3, 3, -1, 3, -1, 3, 0, 1], [12, -7, -3, -1, 3, -1, 3, 3, 0, -7], [-3, -13, 0, 5, -3, 16, 8, 11, 3, 2], [4, -1, -8, -12, 2, -9, 5, -5, -4, -2], [5, -13, 6, 1, 0, -7, -6, 0, 5, -7], [0, -2, -8, 8, 0, 6, 2, 6, 0, 0], [-4, 14, 6, 4, 4, 12, -4, 2, 0, 6], [14, -7, -1, 2, -3, -11, -6, 10, 14, 2], [1, -8, 8, 12, 0, 16, 4, 12, -1, 4], [-2, -8, -9, -1, -4, 3, 0, -5, 2, 4], [-4, 7, 12, 14, -4, 9, -9, 7, 4, -2], [0, -2, 6, 2, -6, 2, 2, 2, -18, -2], [22, -4, 0, -2, 4, 0, 2, 2, 0, -6], [6, 12, -17, 7, 15, 10, 2, 0, 6, -5], [14, 12, 9, 9, 0, -3, -12, -3, -14, 0], [-20, -2, -6, 4, -8, -4, 0, -2, 0, 6], [-7, -13, -5, -3, 2, 6, 11, 4, 7, 4], [0, 6, -12, -14, 0, 6, 8, -4, 6, 2], [-3, 11, 11, 5, 8, -8, -5, -2, 3, 2], [5, 9, 2, -9, -5, 6, 2, 1, 5, 12], [0, -7, 0, -1, 5, 2, -6, -1, 10, 6], [0, 0, -6, 0, 8, -2, -6, -4, 0, 2], [-3, 3, 2, -3, 6, 11, 2, -10, -3, -5], [-13, -9, -8, -7, 3, -2, 8, -3, 13, 4], [0, 13, -2, 7, -3, -8, 4, -1, -2, -12], [-12, 13, 9, -1, 7, 11, -5, 3, 0, 1], [0, -3, -8, -13, 1, 10, 2, -5, -20, 6], [8, 3, 7, -7, -7, -11, -3, -11, 0, 7], [0, 1, -4, -5, -3, 6, 2, -5, 4, -2], [0, 12, -8, -2, 4, -8, 6, 2, -8, -2], [-26, 4, 5, 7, 0, 1, -6, -1, 26, 2], [-12, 0, 6, -4, 2, -4, 2, -6, 0, 0], [-2, 0, 5, 1, -4, 9, 4, 13, 2, -8], [-19, 3, -3, -3, 0, -6, -3, -6, 19, 0], [0, -3, 8, 7, 1, -6, -2, 7, 6, 2], [-2, -7, 15, -8, -6, 2, 0, -9, -2, -1], [0, -2, 16, 14, -6, -8, 0, 10, 4, -4], [8, 14, 7, 3, 2, -9, -10, -5, -8, -2], [-3, -1, 7, 1, 8, 0, 7, 6, 3, 2], [0, -3, -1, -7, 3, 1, 3, 5, -8, 9], [6, -6, 6, -6, -2, -14, 6, -14, 0, 2]]
